@@ -576,8 +576,14 @@ static gboolean
 activate_window (GtkWidget *widget)
 {
   gint active;
+  TaskItemPrivate *priv;
 
   g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  g_return_val_if_fail (TASK_IS_ITEM (widget), FALSE);
+
+  priv = TASK_ITEM (widget)->priv;
+  
+  g_return_val_if_fail (WNCK_IS_WINDOW (priv->window), FALSE);  
 
   active = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "drag-true"));
 
@@ -585,7 +591,7 @@ activate_window (GtkWidget *widget)
   { 
     WnckWindow *window;
 
-    window = (WnckWindow*)g_object_get_data (G_OBJECT (widget), "WnckWindow");
+    window = priv->window;
     if (WNCK_IS_WINDOW (window))
       wnck_window_activate (window, time (NULL));
   }
@@ -761,6 +767,10 @@ task_item_new (WnckWindow *window)
                     G_CALLBACK (on_enter_notify), item);
   g_signal_connect (item, "leave-notify-event",
                     G_CALLBACK (on_leave_notify), item);
+  g_signal_connect (item, "drag-motion",
+                    G_CALLBACK (on_drag_motion), item);
+  g_signal_connect (item, "drag-leave",
+                    G_CALLBACK (on_drag_leave), item);
                     
   task_item_set_visibility (task);
   task_item_setup_atk (task);
