@@ -43,12 +43,13 @@ struct _TaskTitlePrivate
   WnckWindow *window;
   GtkWidget *align;
   GtkWidget *box;
-  GtkWidget *image;
   GtkWidget *label;
   GtkWidget *button;
   GtkWidget *button_image;
+  GdkPixbuf *quit_icon;
 
   gboolean show_home_title;
+  gboolean mouse_in_close_button;
 };
 
 static void disconnect_window (TaskTitle *title);
@@ -64,7 +65,7 @@ on_close_clicked (GtkButton *button,
   g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
   priv = title->priv;
 
-  if (event->button != 1)
+  if (event->button != 1 || !priv->mouse_in_close_button)
     return FALSE;
 
   window = wnck_screen_get_active_window (priv->screen);
@@ -86,6 +87,60 @@ on_close_clicked (GtkButton *button,
   return TRUE;
 }
 
+static gboolean
+on_enter_notify (GtkWidget *widget,
+                 GdkEventCrossing *event,
+                 TaskTitle *title)
+{
+  g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
+  
+  title->priv->mouse_in_close_button = TRUE;
+  gtk_widget_queue_draw (widget);
+  
+  return FALSE;
+}
+
+static gboolean
+on_leave_notify (GtkWidget *widget,
+                 GdkEventCrossing *event,
+                 TaskTitle *title)
+{
+  g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
+  
+  title->priv->mouse_in_close_button = FALSE;
+  gtk_widget_queue_draw (widget);
+
+  return FALSE;
+}
+
+static gboolean
+on_button_expose (GtkWidget *widget, 
+                  GdkEventExpose *event,
+                  TaskTitle *title)
+{
+  g_return_val_if_fail (TASK_IS_TITLE (title), FALSE);
+
+  TaskTitlePrivate *priv;
+  priv = title->priv;
+  
+  if (priv->mouse_in_close_button)
+  {
+    GtkStyle *style = gtk_widget_get_style (widget);
+    gtk_paint_box (style, 
+                   event->window,
+                   GTK_STATE_PRELIGHT,
+                   GTK_SHADOW_NONE,
+                   NULL,
+                   NULL,
+                   NULL,
+                   event->area.x,
+                   event->area.y + 2,
+                   event->area.width,
+                   event->area.height - 4); 
+  }
+  return FALSE;
+}
+
 static void
 on_name_changed (WnckWindow *window, TaskTitle *title)
 {
@@ -118,8 +173,6 @@ on_icon_changed (WnckWindow *window, TaskTitle *title)
   if (priv->window != window)
     return;
 
-  gtk_image_set_from_pixbuf (GTK_IMAGE (title->priv->image),
-                             wnck_window_get_mini_icon (window));
   gtk_widget_queue_draw (GTK_WIDGET (title));
 }
 
@@ -195,11 +248,9 @@ on_active_window_changed (WnckScreen *screen,
   { 
     if (priv->show_home_title)
     {
-      gtk_image_set_from_stock (GTK_IMAGE (priv->image), 
-                                GTK_STOCK_HOME, GTK_ICON_SIZE_MENU);
       gtk_label_set_text (GTK_LABEL (priv->label), _("Home"));
-      gtk_image_set_from_stock (GTK_IMAGE (priv->button_image), 
-                                GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU);
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->button_image), 
+                                 priv->quit_icon);
       gtk_widget_set_tooltip_text (priv->button,
                                 _("Log off, switch user, lock screen or power "
                                      "down the computer"));
@@ -212,13 +263,10 @@ on_active_window_changed (WnckScreen *screen,
       gtk_widget_set_tooltip_text (priv->button, NULL);
       gtk_widget_set_tooltip_text (GTK_WIDGET (title), NULL);
       gtk_widget_hide (priv->box);
-      gtk_widget_hide (priv->box);
     }
   }
   else
   {
-    gtk_image_set_from_pixbuf (GTK_IMAGE (priv->image), 
-                               wnck_window_get_mini_icon (act_window));
     gtk_label_set_text (GTK_LABEL (priv->label), 
                         wnck_window_get_name (act_window));
     gtk_image_set_from_stock (GTK_IMAGE (priv->button_image), 
@@ -250,11 +298,9 @@ on_active_window_changed (WnckScreen *screen,
     if (task_list_get_desktop_visible (TASK_LIST (task_list_get_default ()))
         && priv->show_home_title)
     {
-      gtk_image_set_from_stock (GTK_IMAGE (priv->image), 
-                                GTK_STOCK_HOME, GTK_ICON_SIZE_MENU);
       gtk_label_set_text (GTK_LABEL (priv->label), _("Home"));
-      gtk_image_set_from_stock (GTK_IMAGE (priv->button_image), 
-                                GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU);
+      gtk_image_set_from_pixbuf (GTK_IMAGE (priv->button_image), 
+                                 priv->quit_icon);
       gtk_widget_set_tooltip_text (priv->button,
                                 _("Log off, switch user, lock screen or power "
                                      "down the computer"));
@@ -288,17 +334,27 @@ on_button_release (GtkWidget *title, GdkEventButton *event)
 
   window = wnck_screen_get_active_window (priv->screen);
 
-  if (event->button != 3)
-    return FALSE;
+  g_return_val_if_fail (WNCK_IS_WINDOW (window), FALSE);
 
-  if (!WNCK_IS_WINDOW (window)
-      || wnck_window_get_window_type (window) == WNCK_WINDOW_DESKTOP)
-    return FALSE;
-
-  menu = wnck_action_menu_new (window);
-  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
-                  event->button, event->time);
-  return TRUE;
+  if (event->button == 3)
+  {
+    if (wnck_window_get_window_type (window) != WNCK_WINDOW_DESKTOP)
+    {
+      menu = wnck_action_menu_new (window);
+      gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 
+                      event->button, event->time);
+      return TRUE;
+    }
+  }
+  else if (event->button == 1)
+  {
+    if (event->type == GDK_2BUTTON_PRESS && wnck_window_is_maximized (window))
+    {
+      wnck_window_unmaximize (window);
+    }
+  }
+  
+  return FALSE;
 }
 
 
@@ -329,6 +385,8 @@ task_title_finalize (GObject *object)
   priv = TASK_TITLE_GET_PRIVATE (object);
   disconnect_window (TASK_TITLE (object));
 
+  g_object_unref (G_OBJECT (priv->quit_icon));
+
   G_OBJECT_CLASS (task_title_parent_class)->finalize (object);
 }
 
@@ -349,6 +407,12 @@ task_title_init (TaskTitle *title)
 {
   TaskTitlePrivate *priv;
   GConfClient *client;
+  GdkScreen *gdkscreen;
+  GtkIconTheme *theme;
+  GtkSettings *settings;
+  GdkPixbuf *pixbuf;
+  AtkObject *atk;
+  int width, height;
     	
   priv = title->priv = TASK_TITLE_GET_PRIVATE (title);
 
@@ -373,13 +437,15 @@ task_title_init (TaskTitle *title)
   gtk_widget_set_no_show_all (priv->box, TRUE);
   gtk_widget_show (priv->box);
 
-  priv->image = gtk_image_new_from_stock (GTK_STOCK_HOME, GTK_ICON_SIZE_MENU);
-  gtk_box_pack_start (GTK_BOX (priv->box), priv->image, FALSE, FALSE, 0);
-  gtk_widget_show (priv->image);
-
   priv->label = gtk_label_new (_("Home"));
   gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
   gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
+  
+  PangoAttrList *attr_list = pango_attr_list_new ();
+  PangoAttribute *attr = pango_attr_weight_new (PANGO_WEIGHT_BOLD);
+  pango_attr_list_insert (attr_list, attr);
+  gtk_label_set_attributes (GTK_LABEL (priv->label), attr_list);
+  
   gtk_box_pack_start (GTK_BOX (priv->box), priv->label, TRUE, TRUE, 0);
   gtk_widget_show (priv->label);
 
@@ -389,11 +455,41 @@ task_title_init (TaskTitle *title)
                                NULL);
   gtk_box_pack_start (GTK_BOX (priv->box), priv->button, FALSE, FALSE, 0);
   gtk_widget_show (priv->button);
+  
+  atk = gtk_widget_get_accessible (priv->button);
+  atk_object_set_name (atk, _("Close"));
+  atk_object_set_description (atk, _("Close current window."));
+  atk_object_set_role (atk, ATK_ROLE_PUSH_BUTTON);
+  
   g_signal_connect (priv->button, "button-release-event", 
                     G_CALLBACK (on_close_clicked), title);
+  g_signal_connect (priv->button, "enter-notify-event",
+                    G_CALLBACK (on_enter_notify), title);
+  g_signal_connect (priv->button, "leave-notify-event",
+                    G_CALLBACK (on_leave_notify), title);
+  g_signal_connect (priv->button, "expose-event",
+                    G_CALLBACK (on_button_expose), title);
 
-  priv->button_image = gtk_image_new_from_stock (GTK_STOCK_QUIT, 
-                                                 GTK_ICON_SIZE_MENU);
+  /* Load the quit icon.  We have to do this in such a god-forsaken way
+     because of http://bugzilla.gnome.org/show_bug.cgi?id=581359 and the
+     fact that we support as far back as GTK+ 2.12 (which never passes
+     FORCE_SIZE in GtkImage).  The only way to guarantee icon size is to
+     load and scale it ourselves.  We don't do this for all the other icons
+     in the source just because we know that this icon doesn't come in the
+     size we want. */
+  gdkscreen = gtk_widget_get_screen (GTK_WIDGET (title));
+  theme = gtk_icon_theme_get_for_screen (gdkscreen);
+  settings = gtk_settings_get_for_screen (gdkscreen);
+  if (!gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU,
+&width, &height))
+      width = height = 16;
+  width = MIN(width, height);
+  pixbuf = gtk_icon_theme_load_icon (theme, "gnome-logout", width, 0, NULL);
+  priv->quit_icon = gdk_pixbuf_scale_simple (pixbuf, width, width,
+                                             GDK_INTERP_BILINEAR);
+  g_object_unref (G_OBJECT (pixbuf));
+
+  priv->button_image = gtk_image_new_from_pixbuf (priv->quit_icon);
   gtk_container_add (GTK_CONTAINER (priv->button), priv->button_image);
   gtk_widget_show (priv->button_image);
 
